/* Copyright (c) 2003-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

if (typeof(ko)=='undefined') {
    var ko = {};
}
xtk.include("color");

/**
 * This module defines how Scintilla markers are used in Komodo.
 * http://www.scintilla.org/ScintillaDoc.html#Markers
 *
 * Marker are the images in the editor gutter for things like bookmarks,
 * breakpoints. Also line background color for current line, current line in a
 * debugger session, etc.
 */
ko.markers =  function markers_module() {
    // private vars
    var content_cache = {};
    
    return {
    
    // Marker numbers.
    // Most of the ISciMoz.marker*() methods take a marker number argument.
    // The higher the marker number the higher the marker's z-order.
    // Here are the marker numbers that Komodo uses.
    MARKNUM_HISTORYLOC: 13,
    MARKNUM_STDERR: 12, // used in terminal view
    MARKNUM_STDOUT: 11, // used in terminal view
    MARKNUM_CURRENT_LINE_BACKGROUND: 10,
    MARKNUM_STDIN_PROMPT: 9, // used in terminal view
    MARKNUM_INTERACTIVE_PROMPT_MORE: 8, // used in interactive shell
    MARKNUM_INTERACTIVE_PROMPT: 7, // used in interactive shell
    MARKNUM_BOOKMARK: 6,
    MARKNUM_DEBUG_CURRENT_LINE: 5,
    MARKNUM_SPAWNPOINT_ENABLED: 4,
    MARKNUM_SPAWNPOINT_DISABLED: 3,
    MARKNUM_BREAKPOINT_ENABLED: 2,
    MARKNUM_BREAKPOINT_DISABLED: 1,
    MARKNUM_TRANSIENTMARK: 0, // used in buffer view
    
    // Include all markers *except* MARKNUM_CURRENT_LINE_BACKGROUND.
    // - Want the Nth bitfield for marker N to be set iff that marker should
    //   up in the symbol margin and cleared if the line background should be
    //   colored.
    MARKERS_MASK_SYMBOLS: 0x03FF,
    
    /**
     * Read a file from disk, cache and return the contents.
     *
     * @param {String} uri file uri
     * @param {boolean} force force read from file
     * 
     * Note: The file contents are cached by URI.
     * This is used to load pixmaps for scintilla markers.
     */
    getPixmap: function(uri, force) {
        if (!force && typeof(content_cache[uri]) != 'undefined') {
            return content_cache[uri];
        }
        var file = Components.classes["@activestate.com/koFileEx;1"]
                .createInstance(Components.interfaces.koIFileEx)
        file.URI = uri;
        file.open('rb');
        content_cache[uri] = file.readfile();
        file.close();
        return content_cache[uri];
    },

    /**
     * Setup the standard Komodo markers in the given SciMoz instance and
     * return an appropriate mask for ISciMoz.setMarginMaskN(<n>, <mask>).
     * 
     * @param {Components.interfaces.ISciMoz} scimoz - A plugin instance.
     */
    setup: function(scimoz) {
        //XXX Was this: scimoz.markerDefine(ko.markers.MARKNUM_CURRENT_LINE_BACKGROUND, 32);
        //scimoz.markerDefine(ko.markers.MARKNUM_CURRENT_LINE_BACKGROUND,
        //                    scimoz.SC_MARK_BACKGROUND);
        //XXX The colors for this marker are changed depending on if we are
        //    debugging or not and what the user's Komodo style is.
        scimoz.markerDefine(ko.markers.MARKNUM_BOOKMARK, scimoz.SC_MARK_ARROWDOWN);
        scimoz.markerSetFore(ko.markers.MARKNUM_BOOKMARK, xtk.color.RGB(0x00, 0x00, 0x00)); // black
        scimoz.markerSetBack(ko.markers.MARKNUM_BOOKMARK, xtk.color.RGB(0x00, 0xFF, 0xFF)); // cyan
    
        scimoz.markerDefine(ko.markers.MARKNUM_DEBUG_CURRENT_LINE, scimoz.SC_MARK_SHORTARROW);
        scimoz.markerSetFore(ko.markers.MARKNUM_DEBUG_CURRENT_LINE, xtk.color.RGB(0x8f, 0x00, 0x00));
        scimoz.markerSetBack(ko.markers.MARKNUM_DEBUG_CURRENT_LINE, xtk.color.yellow);
    
        scimoz.markerDefinePixmap(ko.markers.MARKNUM_SPAWNPOINT_ENABLED,
                                  ko.markers.getPixmap("chrome://komodo/skin/images/spawnpoint.xpm"));
        scimoz.markerDefinePixmap(ko.markers.MARKNUM_SPAWNPOINT_DISABLED,
                                  ko.markers.getPixmap("chrome://komodo/skin/images/spawnpoint_disable.xpm"));
        scimoz.markerDefinePixmap(ko.markers.MARKNUM_BREAKPOINT_ENABLED,
                                  ko.markers.getPixmap("chrome://komodo/skin/images/breakpoint_enabled.xpm"));
        scimoz.markerDefinePixmap(ko.markers.MARKNUM_BREAKPOINT_DISABLED,
                                  ko.markers.getPixmap("chrome://komodo/skin/images/breakpoint_disabled.xpm"));
        scimoz.markerDefinePixmap(ko.markers.MARKNUM_INTERACTIVE_PROMPT,
                                  ko.markers.getPixmap("chrome://komodo/skin/images/prompt.xpm"));
    
        scimoz.markerDefine(ko.markers.MARKNUM_INTERACTIVE_PROMPT_MORE, scimoz.SC_MARK_DOTDOTDOT);
        scimoz.markerSetFore(ko.markers.MARKNUM_INTERACTIVE_PROMPT_MORE, xtk.color.black);
    
        scimoz.markerDefine(ko.markers.MARKNUM_STDIN_PROMPT, scimoz.SC_MARK_CHARACTER+'%'.charCodeAt(0));
        scimoz.markerSetFore(ko.markers.MARKNUM_STDIN_PROMPT, xtk.color.red);
    
        scimoz.markerDefine(ko.markers.MARKNUM_STDOUT, scimoz.SC_MARK_EMPTY);
        //scimoz.markerSetBack(ko.markers.MARKNUM_STDOUT, xtk.color.RGB(0xFA, 0xFA, 0xFF));
        scimoz.markerDefine(ko.markers.MARKNUM_STDERR, scimoz.SC_MARK_EMPTY);
        //scimoz.markerSetBack(ko.markers.MARKNUM_STDERR, xtk.color.RGB(0xFF, 0xFA, 0xFA));
        scimoz.markerDefine(ko.markers.MARKNUM_HISTORYLOC, scimoz.SC_MARK_EMPTY);
    
        //XXX Have to see if this is necessary, i.e. if anything helpful is
        //    being done with this.
        //// Failed breakpoints (mo.fb not at a breakable position)
        //scimoz.markerDefine(1, scimoz.SC_MARK_CIRCLE);
        //scimoz.markerSetFore(1, RGB (0x8f, 0x00, 0x00));
        //scimoz.markerSetBack(1, xtk.color.green);
    
        scimoz.markerDefine(ko.markers.MARKNUM_TRANSIENTMARK, scimoz.SC_MARK_EMPTY);
    
        return ko.markers.MARKERS_MASK_SYMBOLS;
    }
    };

}();

/**
 * @deprecated since 7.0
 */
ko.logging.globalDeprecatedByAlternative("markers_setup", "ko.markers.setup");
ko.logging.globalDeprecatedByAlternative("markers_getPixmap", "ko.markers.getPixmap");
ko.logging.globalDeprecatedByAlternative("open_readPixmap", "ko.markers.getPixmap");
ko.logging.globalDeprecatedByAlternative("MARKNUM_TRANSIENTMARK", "ko.markers.MARKNUM_TRANSIENTMARK");
ko.logging.globalDeprecatedByAlternative("MARKNUM_BREAKPOINT_DISABLED", "ko.markers.MARKNUM_BREAKPOINT_DISABLED");
ko.logging.globalDeprecatedByAlternative("MARKNUM_BREAKPOINT_ENABLED", "ko.markers.MARKNUM_BREAKPOINT_ENABLED");
ko.logging.globalDeprecatedByAlternative("MARKNUM_SPAWNPOINT_DISABLED", "ko.markers.MARKNUM_SPAWNPOINT_DISABLED");
ko.logging.globalDeprecatedByAlternative("MARKNUM_SPAWNPOINT_ENABLED", "ko.markers.MARKNUM_SPAWNPOINT_ENABLED");
ko.logging.globalDeprecatedByAlternative("MARKNUM_DEBUG_CURRENT_LINE", "ko.markers.MARKNUM_DEBUG_CURRENT_LINE");
ko.logging.globalDeprecatedByAlternative("MARKNUM_BOOKMARK", "ko.markers.MARKNUM_BOOKMARK");
ko.logging.globalDeprecatedByAlternative("MARKNUM_INTERACTIVE_PROMPT", "ko.markers.MARKNUM_INTERACTIVE_PROMPT");
ko.logging.globalDeprecatedByAlternative("MARKNUM_INTERACTIVE_PROMPT_MORE", "ko.markers.MARKNUM_INTERACTIVE_PROMPT_MORE");
ko.logging.globalDeprecatedByAlternative("MARKNUM_STDIN_PROMPT", "ko.markers.MARKNUM_STDIN_PROMPT");
ko.logging.globalDeprecatedByAlternative("MARKNUM_CURRENT_LINE_BACKGROUND", "ko.markers.MARKNUM_CURRENT_LINE_BACKGROUND");
ko.logging.globalDeprecatedByAlternative("MARKNUM_STDOUT", "ko.markers.MARKNUM_STDOUT");
ko.logging.globalDeprecatedByAlternative("MARKNUM_STDERR", "ko.markers.MARKNUM_STDERR");
